precision lowp float;
//纹理图片
uniform sampler2D displayTexture;
//纹理坐标
varying lowp vec2 varyTextureCoord;
//颜色矩阵 更改亮度
uniform lowp mat4 colorMatrix;
// 新的发色
uniform vec4 hairColor;
// 是否显示原图 0.0 显示原图 否则按发色进行染色
uniform lowp float showColorHair;
// 是否使用自定义色相
uniform lowp float usingHSL;
// 自定义色相及饱和度值
uniform vec2 hueAndSaturation;
uniform lowp float intensity;
uniform lowp float gamma;
uniform lowp float colorSaturation;

#ifndef saturate
#define saturate(v) clamp(v, 0., 1.)
#endif
highp vec3 hue2rgb(float hue){
    hue=fract(hue);
    return saturate(vec3(
    abs(hue*6.-3.)-1.,
    2.-abs(hue*6.-2.),
    2.-abs(hue*6.-4.)
    ));
}
highp vec3 hsl2rgb(vec3 hsl){
    if (hsl.y==0.){
        return vec3(hsl.z);//Luminance.
    } else {
        float b;
        if (hsl.z<.5){
            b=hsl.z*(1.+hsl.y);
        } else {
            b=hsl.z+hsl.y-hsl.y*hsl.z;
        }
        float a=2.*hsl.z-b;
        return a+hue2rgb(hsl.x)*(b-a);
    }
}
highp vec3 rgb2hsl(vec3 c){
    float  cMin=min(min(c.r, c.g), c.b);
    float  cMax=max(max(c.r, c.g), c.b);
    float  delta=cMax-cMin;
    highp  vec3 hsl=vec3(0., 0., (cMax+cMin)/2.);
    if (delta!=0.0){ //If it has chroma and isn't gray.
        if (hsl.z<.5){
            hsl.y=delta/(cMax+cMin);//Saturation.
        } else {
            hsl.y=delta/(2.-cMax-cMin);//Saturation.
        }
        float deltaR=(((cMax-c.r)/6.)+(delta/2.))/delta;
        float    deltaG=(((cMax-c.g)/6.)+(delta/2.))/delta;
        float    deltaB=(((cMax-c.b)/6.)+(delta/2.))/delta;
        //Hue.
        if (c.r==cMax){
            hsl.x=deltaB-deltaG;
        } else if (c.g==cMax){
            hsl.x=(1./3.)+deltaR-deltaB;
        } else { //if(c.b==cMax){
            hsl.x=(2./3.)+deltaG-deltaR;
        }
        hsl.x=fract(hsl.x);
    }
    return hsl;
}

void main(){

    //尝试在这里进行发色更改
    lowp vec4 textureColor = texture2D(displayTexture, varyTextureCoord);
    lowp vec4 oColor = textureColor;
    if (showColorHair != 0.0) {
        //gamma
        textureColor = vec4(pow(textureColor.rgb, vec3(gamma)), textureColor.w);
        //如果不显示原图 那么就是染发
        //先将本来的发色转为HSL 然后更改H,S至目标色的H,S ，再转回RGB
        highp vec3 colorHSL = rgb2hsl(textureColor.rgb);
        highp vec3 desColorHSL = rgb2hsl(hairColor.rgb);
        if (hairColor.x == 0. && hairColor.y == 0. && hairColor.z == 0.) {
            desColorHSL = rgb2hsl(vec3(0.0, 0.0, 0.0));
            colorHSL.x = 0.93;
            colorHSL.z = colorHSL.z * 0.4;
            colorHSL.y = 0.06;
            colorHSL.z = clamp(colorHSL.z, 0.0, 1.0);
        } else if (hairColor.x == 1. && hairColor.y == 1. && hairColor.z == 1.) {
            desColorHSL = rgb2hsl(vec3(1.0, 1.0, 1.0));
            colorHSL.x = 0.;
            colorHSL.z = colorHSL.z * 1.8;
            colorHSL.y = 0.;
            colorHSL.z = clamp(colorHSL.z, 0.0, 1.0);
        } else {
            colorHSL.x = desColorHSL.x;
            colorHSL.y = desColorHSL.y;
            colorHSL.y = colorHSL.y + colorSaturation;
            if (colorHSL.y < 0.0) {
                colorHSL.y = 0.0;
            }
            if (colorHSL.y > 1.0) {
                colorHSL.y = 1.0;
            }
        }

        highp vec3 finalColorRGB = hsl2rgb(colorHSL);
        textureColor = vec4(finalColorRGB, textureColor.w);
        /// 强度调整
        vec4 lastColor  = (intensity * textureColor) * colorMatrix + ((1.0 - intensity) * oColor);
        gl_FragColor = lastColor;
        return;
    }

    if (usingHSL == 1.0) {
        //gamma
        textureColor = vec4(pow(textureColor.rgb, vec3(gamma)), textureColor.w);

        highp vec3 colorHSL = rgb2hsl(textureColor.rgb);
        colorHSL.x = hueAndSaturation.x;
        colorHSL.y = hueAndSaturation.y;
        colorHSL.y = colorHSL.y + colorSaturation;
        if (colorHSL.y < 0.0) {
            colorHSL.y = 0.0;
        }
        if (colorHSL.y > 1.0) {
            colorHSL.y = 1.0;
        }
        highp vec3 finalColorRGB = hsl2rgb(colorHSL);
        textureColor = vec4(finalColorRGB, textureColor.w);
        /// 强度调整
        vec4 lastColor  = (intensity * textureColor) * colorMatrix + ((1.0 - intensity) * oColor);
        gl_FragColor = lastColor;
        return;
    }

    gl_FragColor = vec4(textureColor.r, textureColor.g, textureColor.b, textureColor.a);
}
